@@ -162,23 +162,18 @@ r_to_py.error <- function(x, convert = FALSE) {
   e
 }
 
-#' @export
-conditionCall.python.builtin.BaseException <- function(c) {
-  as_r_value(py_get_attr(c, "call", TRUE))
-}
-
-#' @export
-conditionMessage.python.builtin.BaseException <- function(c) {
-  conditionMessage_from_py_exception(c)
-}
-
 
 #' @export
 `$.python.builtin.BaseException` <- function(x, name) {
-  if(identical(name, "call"))
-    return(conditionCall(x))
-  if(identical(name, "message"))
-    return(conditionMessage(x))
+  if(identical(name, "call")) {
+    out <- if(typeof(x) == "list") unclass(x)[["call"]]
+    return(out %||% as_r_value(py_get_attr(x, "call", TRUE)))
+  }
+
+  if(identical(name, "message")) {
+    out <- if(typeof(x) == "list") unclass(x)[["message"]]
+    return(out %||% conditionMessage_from_py_exception(x))
+  }
 
   py_maybe_convert(py_get_attr(x, name, TRUE), py_has_convert(x))
 }
@@ -246,16 +241,19 @@ defer <- function(expr, envir = parent.frame()) {
 
 #' @importFrom utils head
 disable_conversion_scope <- function(object) {
+  # Though this is not part of the exported API, there are external packages
+  # that reach in with ::: to use this function. (e.g., {zellkonverter} on
+  # Bioconductor). Take care that symbols like `py_set_convert` and `object`
+  # don't need to be in the on.exit() expression search path.
 
-  if (!inherits(object, "python.builtin.object"))
+  if (!is_py_object(object) || !py_get_convert(object))
     return(FALSE)
 
-  envir <- as.environment(object)
-  if (exists("convert", envir = envir, inherits = FALSE)) {
-    convert <- get("convert", envir = envir)
-    assign("convert", FALSE, envir = envir)
-    defer(assign("convert", convert, envir = envir), envir = parent.frame())
-  }
+  envir <- parent.frame()
+  cl <- as.call(c(py_set_convert, object, TRUE))
+
+  py_set_convert(object, FALSE)
+  do.call(on.exit, list(cl, add = TRUE), envir = envir)
 
   TRUE
 }
