@@ -145,3 +145,47 @@ test_that("numpy string arrays are correctly handled", {
                                "17", "18"), byrow = TRUE, ncol = 2))
 
   })
+
+
+test_that("numpy non-simple arrays work", {
+  # https://github.com/rstudio/reticulate/issues/1613
+  py_run_string("import numpy as np", convert = FALSE)
+  py_run_string(
+    "array = np.array([(1.0, 2), (3.0, 4)], dtype=[('x', float), ('y', int)])",
+    convert = FALSE
+  )
+  result <- py_run_string("rec_array = array.view(np.recarray)", convert = FALSE)
+
+  # Test that attempting to convert a non-simple array fails gracefully,
+  # returns a PyObjectRef.
+  rec_array <- py_to_r(result$rec_array)
+                                      # numpy 1.0 vs 2.0
+  expect_s3_class(rec_array, c("numpy.recarray", "numpy.rec.recarray"))
+  expect_s3_class(rec_array, "numpy.ndarray")
+  expect_s3_class(rec_array, "python.builtin.object")
+
+  # Test that a registered S3 method for the non-simple numpy array will be
+  # called. (Note, some packages, like {zellkonverter}, will register this
+  # directly for numpy.ndarray)
+  py_to_r.numpy.recarray <- function(x) {
+    tryCatch({
+      pandas <- import("pandas", convert = FALSE)
+      x <- pandas$DataFrame(x)$to_numpy()
+      x <- py_to_r(x)
+      return(x)
+    }, error = identity)
+    NextMethod()
+  }
+  registerS3method("py_to_r", "numpy.recarray", py_to_r.numpy.recarray)
+  registerS3method("py_to_r", "numpy.rec.recarray", py_to_r.numpy.recarray)
+
+  on.exit({
+    rm(list = c("py_to_r.numpy.recarray", "py_to_r.numpy.rec.recarray"),
+       envir = environment(py_to_r)$.__S3MethodsTable__.)
+  })
+
+  arr <- py_to_r(result$rec_array)
+  expect_identical(arr, rbind(c(1, 2),
+                              c(3, 4)))
+
+})
